@@ -1,6 +1,9 @@
-from abstract_utilities import convert_to_percentage,read_from_file,eatAll
+from abstract_utilities import convert_to_percentage,read_from_file,eatAll,make_list
 from abstract_utilities.type_utils import is_number
-from abstract_utilities.parse_utils import num_tokens_from_string,chunk_source_code
+from abstract_utilities.parse_utils import num_tokens_from_string,chunk_source_code,chunk_any_to_tokens
+from ..model_selection.ModelBuilder import ModelManager
+from ..instruction_selection import InstructionManager
+from ..api_selection.ApiBuilder import ApiManager
 class PromptManager:
     """
     Manages the generation and management of prompts. This includes creating prompts based on user input or predefined conditions, formatting prompts, and handling errors or special cases.
@@ -32,11 +35,13 @@ class PromptManager:
             chunk: The current chunk.
             chunk_type: The type of chunk (e.g., URL, SOUP, DOCUMENT, CODE, TEXT).
         """
+        self.notation_text = ''
+        self.prompt_as_previous_text = ''
         self.chunk_type = chunk_type
         
-        self.instruction_mgr = instruction_mgr
+        self.instruction_mgr = instruction_mgr or InstructionManager()
         
-        self.model_mgr=model_mgr
+        self.model_mgr=model_mgr or ModelManager()
         
         self.role=role
         
@@ -45,7 +50,7 @@ class PromptManager:
         self.instruction_data = instruction_data or self.instruction_mgr.instructions
         if not isinstance(self.instruction_data,list):
             self.instruction_data = [self.instruction_data]
-    
+        
         self.request_data=request_data or ['']
         if not isinstance(self.request_data,list):
             self.request_data = [self.request_data]
@@ -190,7 +195,7 @@ class PromptManager:
             """
             # Function implementation ...
             get_token_distributions = []
-            data_chunks_list = chunk_source_code(total_chunk_data, fictitious_chunk_token_length)
+            data_chunks_list = chunk_any_to_tokens(total_chunk_data, fictitious_chunk_token_length)
             while '' in data_chunks_list:
                 data_chunks_list.remove('')
             total_chunks = len(data_chunks_list)
@@ -238,8 +243,10 @@ class PromptManager:
             
             prompt_percent = convert_to_percentage(100-tokenize_js["completion_percentage"])
             prompt_token_desired = int(max_tokens*prompt_percent)
-            bot_notation_token_count=int(200)
-            initial_prompt_token_length = num_tokens_from_string(str(total_prompt))+bot_notation_token_count
+            bot_notation_token_count=num_tokens_from_string(str(self.notation_text))
+            prompt_as_previous_token_count = num_tokens_from_string(str(self.prompt_as_previous_text))
+            total_prompt_length = num_tokens_from_string(str(total_prompt))
+            initial_prompt_token_length = total_prompt_length+max(200,bot_notation_token_count)+max(200,prompt_as_previous_token_count)
 
             initial_completion_token_length=bot_notation_token_count
             
@@ -312,9 +319,9 @@ class PromptManager:
             chunk_data= ''
             
         request = request or self.request_data[chunk_token_distribution_number]
-        instructions = instructions or self.instruction_data[chunk_token_distribution_number]['text']
+        self.notation_text = notation
+        self.prompt_as_previous_text=prompt_as_previous
         values_js = {'instructions':{'display':'instructions','data':instructions},
-                     'request_chunks':{'display':'previous chunk data requested','data':prompt_as_previous},
                      'prompt':{'display':'prompt','data':request},
                      'notation':{'display':'notation from the previous response','data':notation},
                      'chunk_data':{'display':'chunk_data','data':get_chunk_header(chunk_number,total_chunks,chunk_data)},
@@ -356,14 +363,49 @@ class PromptManager:
         Returns:
             dict: A dictionary representing the prompt for the chatbot.
         """
-        print(prompt_as_previous)
+        print(chunk_token_distribution_number)
+        print(self.instruction_data)
         self.prompt_as_previous=prompt_as_previous
+        self.notation_text = notation
+        self.prompt_as_previous_text=prompt_as_previous
+        print(prompt_as_previous)
+        if prompt_as_previous and isinstance(prompt_as_previous,str):
+            previous_request_data = self.request_data[chunk_token_distribution_number][:chunk_number]
+            remaining_request_data = self.request_data[chunk_token_distribution_number][chunk_number:]
+            previous_prompt_data = self.prompt_data[chunk_token_distribution_number][:chunk_number]
+            remaining_prompt_data = self.prompt_data[chunk_token_distribution_number][chunk_number:]
+            collated_remaining_prompt_data = '\n'.join(remaining_prompt_data)
+            if chunk_number == 0:
+                previous_chunks = make_list(self.chunk_token_distributions[chunk_token_distribution_number][0])
+                remaining_chunks = make_list(self.chunk_token_distributions[chunk_token_distribution_number][1:])
+            elif chunk_number == len(self.chunk_token_distributions[chunk_token_distribution_number]):
+                previous_chunks = make_list(self.chunk_token_distributions[chunk_token_distribution_number][:-1])
+                remaining_chunks = make_list(self.chunk_token_distributions[chunk_token_distribution_number][-1])
+            else:
+                previous_chunks = make_list(self.chunk_token_distributions[chunk_token_distribution_number][:chunk_number])
+                remaining_chunks = make_list(self.chunk_token_distributions[chunk_token_distribution_number][chunk_number:])
+            collated_chunks = ['\n'.join([chunk.get('chunk',{}).get('data','') for chunk in remaining_chunks])]
+            new_chunk_distributions=self.calculate_token_distribution(
+                 request_data=remaining_request_data,
+                 prompt_data=collated_chunks,
+                 instruction_data=self.instruction_data[chunk_token_distribution_number].get('text') or [],
+                 notation=self.notation,
+                 max_tokens=self.model_mgr.selected_max_tokens,
+                 completion_percentage=self.completion_percentage,
+                 assume_bot_notation=self.notation_text)
+            self.chunk_token_distributions[chunk_token_distribution_number]=previous_chunks +new_chunk_distributions[0]
+            for chunk_token_distribution_number_count,chunk_distribution in enumerate(self.chunk_token_distributions):
+                for chunk_number_count,chunk in enumerate(chunk_distribution):
+                    self.chunk_token_distributions[chunk_token_distribution_number_count][chunk_number_count]["number"]= chunk_number_count
+                    self.chunk_token_distributions[chunk_token_distribution_number_count][chunk_number_count]["total"]= len(chunk_distribution)
+        
         self.prompt =""
         total_chunks =0
         
         for each in self.chunk_token_distributions:
             total_chunks+=len(each)
         chunk_token_distribution_data=self.chunk_token_distributions[chunk_token_distribution_number][chunk_number]
+        
         self.prompt =self.create_prompt_guide(request=self.request_data[chunk_token_distribution_number],
                                               instructions=instructions or self.instruction_data[chunk_token_distribution_number]['text'],
                                               chunk_token_distribution_number=chunk_token_distribution_number,
@@ -373,8 +415,8 @@ class PromptManager:
                                               notation=notation,
                                               generate_title=generate_title,
                                               request_chunks=request_chunks,
-                                              prompt_as_previous=prompt_as_previous)
-        max_tokens = chunk_token_distribution_data['completion']['available']
+                                              prompt_as_previous=prompt_as_previous)    
+        max_tokens = self.model_mgr.selected_max_tokens -num_tokens_from_string(str({"model": self.model_mgr.selected_model_name, "messages": [{"role": self.role or "user", "content":self.prompt }]}))
         self.prompt ={"model": self.model_mgr.selected_model_name, "messages": [{"role": self.role or "user", "content":self.prompt }],"max_tokens": max_tokens}
         blank = True
         for key in self.key_check:
@@ -382,5 +424,6 @@ class PromptManager:
                 blank=False
         if blank:
             self.prompt['blank_prompt']=True
+        
         return self.prompt
 
